unit Unit1;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, Grids;

type
  TForm1 = class(TForm)
    Button1: TButton;
    SG: TStringGrid;
    Button2: TButton;
    Label1: TLabel;
    procedure FormCreate(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure Button1Click(Sender: TObject);
    procedure SGKeyUp(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure Button2Click(Sender: TObject);
    procedure SGMouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
  private
    procedure DrawList;
    { Private declarations }
  public
    { Public declarations }
  end;

  TObj = class
    X, Y: Integer;
    constructor Create(tx, ty: integer);
  end;

  TWall = class(TObj)
    State: String;
    constructor Create(tx, ty: Integer; tstate: String);
  end;

  TMoveWall = class(TWall)
    Direction: String;
    constructor Create(tx, ty: Integer; tdir: string; tstate: String);
  end;

  TObjList = class
    Count: Word;
    Items: array of TObj;
    procedure AddObj(tx, ty: Integer); overload;
    procedure AddObj(tx, ty: Integer; tstate: String); overload;
    procedure AddObj(tx, ty: Integer; tdir: String; tstate: String); overload;
    procedure DeleteObj(index: Integer);
  end;

var
  Form1: TForm1;
  L: TObjList;
  DirectionList: array [0..4] of String =
            ('Up', 'Down', 'Left', 'Right', 'Random');
  StateList: array [0..2] of String =
            ('Burn', 'Life', 'Dead');

implementation

{$R *.dfm}

{ TObjList }

procedure TObjList.AddObj(tx, ty: Integer);
begin
    inc(Count);
    SetLength(Items, Count);
    Items[Count-1] := TObj.Create(tx, ty);
end;

procedure TObjList.AddObj(tx, ty: Integer; tstate: String);
begin
    inc(Count);
    SetLength(Items, Count);
    Items[Count-1] := TWall.Create(tx, ty, tstate);
end;

procedure TObjList.AddObj(tx, ty: Integer; tdir: String; tstate: String);
begin
    inc(Count);
    SetLength(Items, Count);
    Items[Count-1] := TMoveWall.Create(tx, ty, tdir, tstate);
end;

procedure TObjList.DeleteObj(index: Integer);
begin
    if Count = 0 then exit;
    if Count <> 1 then
        Items[index] := Items[High(Items)];
    dec(Count);
    SetLength(Items, Count); 
end;

{ TObj }

constructor TObj.Create(tx, ty: integer);
begin
    X := tx;
    Y := ty;
end;

{ TWall }

constructor TWall.Create(tx, ty: Integer; tstate: String);
begin
    inherited Create(tx, ty);
    State := tstate;
end;

{ TMoveWall }

constructor TMoveWall.Create(tx, ty: Integer; tdir: String; tstate: String);
begin
    inherited Create(tx, ty, tstate);
    Direction := tdir;
end;

/////////////////////////////////////////////////////////

procedure TForm1.FormCreate(Sender: TObject);
begin
    Randomize;
    L := TObjList.Create;
    SG.Cells[0, 0] := 'Index';
    SG.Cells[1, 0] := 'X';
    SG.Cells[2, 0] := 'Y';
    SG.Cells[3, 0] := 'State';
    SG.Cells[4, 0] := 'Dir';
    SG.Cells[5, 0] := 'ClassName';
end;

procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
begin
    L.Free;
end;

procedure TForm1.Button1Click(Sender: TObject);
var
    objtype: Byte;
    x, y, d, s: Integer;
begin
    x := Random(35);
    y := Random(35);
    objtype := Random(3);
    if objtype = 0 then
        L.AddObj(x, y)
    else if objtype = 1 then
    begin
        s := Random(3);
        L.AddObj(x, y, StateList[s])
    end
    else if objtype = 2 then
    begin
        d := Random(5);
        s := Random(3);
        L.AddObj(x, y, DirectionList[d], StateList[s]);
    end;
    DrawList;
end;

procedure TForm1.SGKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
    Label1.Caption := ' : '+IntToStr(SG.Row)+' '+IntToStr(SG.Col);
end;

procedure TForm1.Button2Click(Sender: TObject);
begin
    L.DeleteObj(SG.Row-1);
    DrawList;
end;

procedure TForm1.SGMouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
    Label1.Caption := ' : '+IntToStr(SG.Row)+' '+IntToStr(SG.Col);
end;

procedure TForm1.DrawList;
var
    i: Integer;
    className: String;
    tw: TWall;
    tmw: TMoveWall;
begin
    if L.Count > 0 then
        SG.RowCount := L.Count+1
    else
    begin
        for i:=0 to 5 do
            SG.Cells[i, 1] := '';
    end;
    
    for i:=0 to L.Count-1 do
    begin
        className := L.Items[i].ClassName;
        SG.Cells[0, i+1] := IntToStr(i);
        SG.Cells[1, i+1] := IntToStr(L.Items[i].X);
        SG.Cells[2, i+1] := IntToStr(L.Items[i].Y);
        if className = 'TWall' then
        begin
            tw := L.Items[i] as TWall;
            SG.Cells[3, i+1] := tw.State;
        end;
//        else if className = 'TMoveWall' then
        if L.Items[i].ClassNameIs('TMoveWall') then
        begin
            tmw := L.Items[i] as TMoveWall;
            SG.Cells[3, i+1] := tmw.State;
            SG.Cells[4, i+1] := tmw.Direction;
        end;
        SG.Cells[5, i+1] := className;
    end;
end;

end.
